@@ -848,6 +848,9 @@ void ProduceCalendar(void)
     if (CalMonths) {
         FromDSE(DSEToday, &y, &m, &d);
         DSEToday = DSE(y, m, 1);
+        LocalDSEToday = DSEToday;
+        LocalSysTime = 0;
+        SysTime = 0;
         GenerateCalEntries(-1);
         DidAMonth = 0;
         if (PsCal == PSCAL_LEVEL3) {
@@ -862,8 +865,13 @@ void ProduceCalendar(void)
         }
         return;
     } else {
-        if (MondayFirst) DSEToday -= (DSEToday%7);
-        else             DSEToday -= ((DSEToday+1)%7);
+        if (MondayFirst) {
+            DSEToday -= (DSEToday%7);
+            LocalDSEToday -= (LocalDSEToday%7);
+        } else {
+            DSEToday -= ((DSEToday+1)%7);
+            LocalDSEToday -= ((LocalDSEToday+1)%7);
+        }
 
         GenerateCalEntries(-1);
 
@@ -912,7 +920,7 @@ static void DoCalendarOneWeek(int nleft)
     int y, m, d, done, i, l, wd;
     char buf[128];
     int LinesWritten = 0;
-    int OrigDse = DSEToday;
+    int OrigDse = LocalDSEToday;
 
     InitMoonsAndShades();
 /* Fill in the column entries */
@@ -920,6 +928,7 @@ static void DoCalendarOneWeek(int nleft)
         ColToDay[i] = DayOf(DSEToday);
         GenerateCalEntries(i);
         DSEToday++;
+        LocalDSEToday++;
     }
 
     /* Figure out weekday of first column */
@@ -1021,6 +1030,9 @@ static void DoCalendarOneWeek(int nleft)
     done = 0;
     while (!done) {
         done = WriteOneCalLine(OrigDse, wd);
+        if (done < 0) {
+            break;
+        }
         LinesWritten++;
     }
 
@@ -1205,6 +1217,7 @@ static int WriteCalendarRow(void)
         GenerateCalEntries(i);
         ColToDay[i] = DayOf(DSEToday);
         DSEToday++;
+        LocalDSEToday++;
     }
 
 /* Output the entries */
@@ -1280,6 +1293,9 @@ static int WriteCalendarRow(void)
     done = 0;
     while (!done) {
         done = WriteOneCalLine(OrigDse, wd);
+        if (done < 0) {
+            break;
+        }
         LinesWritten++;
     }
 
@@ -1472,6 +1488,9 @@ static void PrintCentered(char const *s, int width, char const *pad)
 /*  WriteOneCalLine                                            */
 /*                                                             */
 /*  Write a single line.                                       */
+/*  Returns: -1 if no line was written and nothing left to do  */
+/*            0 if a line was written and there is more to do  */
+/*            1 if a line was written and nothing left to do   */
 /*                                                             */
 /***************************************************************/
 static int WriteOneCalLine(int start_dse, int wd)
@@ -1479,6 +1498,18 @@ static int WriteOneCalLine(int start_dse, int wd)
     int done = 1, i;
     int d;
 
+    /* Do nothing if there are no calendar entries at all */
+    d = 0;
+    for (i=0; i<7; i++) {
+        if (CalColumn[i]) {
+            d = 1;
+            break;
+        }
+    }
+    if (!d) {
+        /* Nothing to do and no line was written */
+        return -1;
+    }
     gon();
     DRAW(tb);
     goff();
@@ -1932,6 +1963,17 @@ static int DoCalRem(ParsePtr p, int col)
         return r;
     }
 
+    if (trig.tz != NULL && tim.ttime == NO_TIME) {
+        FreeTrig(&trig);
+        return E_TZ_NO_AT;
+    }
+
+    /* An empty string for time zone is just a missing time zone */
+    if (trig.tz != NULL && !*trig.tz) {
+        free( (void *) trig.tz);
+        trig.tz = NULL;
+    }
+
     if (trig.typ == MSG_TYPE ||
         trig.typ == CAL_TYPE ||
         trig.typ == MSF_TYPE) {
@@ -1951,7 +1993,9 @@ static int DoCalRem(ParsePtr p, int col)
         return E_EOLN;
     }
     if (trig.typ == SAT_TYPE) {
+        EnterTimezone(trig.tz);
         r=DoSatRemind(&trig, &tim, p);
+        ExitTimezone(trig.tz);
         if (r) {
             if (r == E_CANT_TRIG && trig.maybe_uncomputable) {
                 r = OK;
@@ -2019,7 +2063,9 @@ static int DoCalRem(ParsePtr p, int col)
         }
     } else {
         /* Calculate the trigger date */
+        EnterTimezone(trig.tz);
         dse = ComputeTrigger(get_scanfrom(&trig), &trig, &tim, &r, 1);
+        ExitTimezone(trig.tz);
         if (r) {
             if (r == E_CANT_TRIG && trig.maybe_uncomputable) {
                 r = OK;
@@ -2029,6 +2075,9 @@ static int DoCalRem(ParsePtr p, int col)
         }
     }
 
+    /* Adjust trigger date/time to time zone */
+    dse = AdjustTriggerForTimeZone(&trig, dse, &tim);
+
     /* Add to global OMITs if so indicated */
     if (trig.addomit) {
         r = AddGlobalOmit(dse);
@@ -2299,6 +2348,9 @@ static int DoCalRem(ParsePtr p, int col)
         e->nonconst_expr = nonconst_expr;
         e->if_depth = get_if_pointer() - get_base_if_pointer();
         e->trig = trig;
+        if (e->trig.tz) {
+            e->trig.tz = StrDup(e->trig.tz);
+        }
         e->tt = tim;
 #ifdef REM_USE_WCHAR
         e->wc_pos = NULL;
@@ -2395,6 +2447,9 @@ static void WriteSimpleEntryProtocol1(CalEntry const *e)
 void WriteJSONTimeTrigger(TimeTrig const *tt)
 {
     PrintJSONKeyPairTime("time", tt->ttime);
+    if (tt->ttime != tt->ttime_orig) {
+        PrintJSONKeyPairTime("time_in_tz", tt->ttime_orig);
+    }
     PrintJSONKeyPairTime("nexttime", tt->nexttime);
     PrintJSONKeyPairInt("tdelta", tt->delta);
     PrintJSONKeyPairInt("trep", tt->rep);
@@ -2526,6 +2581,10 @@ void WriteJSONTrigger(Trigger const *t, int include_tags)
     PrintJSONKeyPairDate("from", t->from);
     PrintJSONKeyPairInt("priority", t->priority);
     PrintJSONKeyPairDateTime("eventstart", t->eventstart);
+    if (t->eventstart_orig != NO_TIME &&
+        t->eventstart_orig != t->eventstart) {
+        PrintJSONKeyPairDateTime("eventstart_in_tz", t->eventstart_orig);
+    }
     if (t->eventduration != NO_TIME) {
         PrintJSONKeyPairInt("eventduration", t->eventduration);
     }
@@ -2547,6 +2606,7 @@ void WriteJSONTrigger(Trigger const *t, int include_tags)
         }
         PrintJSONKeyPairString("tags", DBufValue(&(t->tags)));
     }
+    PrintJSONKeyPairString("tz", t->tz);
 }
 
 static void WriteSimpleEntryProtocol2(CalEntry *e)
@@ -2569,6 +2629,9 @@ static void WriteSimpleEntryProtocol2(CalEntry *e)
     }
     if (e->time != NO_TIME) {
         PrintJSONKeyPairInt("time", e->time);
+        if (e->tt.ttime_orig != e->tt.ttime) {
+            PrintJSONKeyPairInt("time_in_tz", e->tt.ttime_orig);
+        }
         if (e->tt.delta) {
             PrintJSONKeyPairInt("tdelta", e->tt.delta);
         }
