@@ -18,6 +18,7 @@
 #include <string.h>
 
 #include <stdlib.h>
+#include <time.h>
 
 #include "types.h"
 #include "globals.h"
@@ -32,6 +33,126 @@ static int ParseUntil (ParsePtr s, Trigger *t, int type);
 static int ShouldTriggerBasedOnWarn (Trigger const *t, int dse, int *err);
 static int ComputeTrigDuration(TimeTrig const *t);
 
+static int CalledEnterTimezone = 0;
+
+int AdjustTriggerForTimeZone(Trigger *trig, int dse, TimeTrig *tim)
+{
+    int y, m, d, hour, minute;
+    int r;
+    struct tm tm;
+    if (!trig->tz || dse < 0) {
+        /* Already local time or did not compute trigger date - no adjustments needed */
+        return dse;
+    }
+    FromDSE(dse, &y, &m, &d);
+    hour = tim->ttime_orig / 60;
+    minute = tim->ttime_orig % 60;
+
+    r = tz_convert(y, m, d, hour, minute, trig->tz, LocalTimeZone, &tm);
+    if (r != 1) {
+        Wprint(tr("Error adjusting trigger to local time zone"));
+        return dse;
+    }
+
+    dse = DSE(tm.tm_year+1900, tm.tm_mon, tm.tm_mday);
+    tim->ttime = tm.tm_hour * 60 + tm.tm_min;
+    /* Adjust eventstart also */
+    trig->eventstart = dse * MINUTES_PER_DAY + tim->ttime;
+    SaveAllTriggerInfo(trig, tim, dse, tim->ttime, 1);
+    if (DebugFlag & DB_PRTTRIG) {
+        fprintf(ErrFp, "%s(%s): Trig(tz_adj %s) = %s, %d %s, %d AT %02d:%02d",
+                GetCurrentFilename(), line_range(LineNoStart, LineNo), trig->tz,
+                get_day_name(dse % 7), tm.tm_mday, get_month_name(tm.tm_mon),
+                1900 + tm.tm_year, tim->ttime / 60, tim->ttime % 60);
+        if (tim->duration != NO_TIME) {
+            fprintf(ErrFp, " DURATION %02d:%02d",
+                    (tim->duration / 60),
+                    (tim->duration % 60));
+        }
+        fprintf(ErrFp, "\n");
+    }
+    return dse;
+}
+
+void ExitTimezone(char const *tz)
+{
+    if (!CalledEnterTimezone) {
+        fprintf(stderr, "ExitTimezone called without EnterTimezone!!!\n");
+        abort();
+    }
+    CalledEnterTimezone = 0;
+    if (!tz || !*tz) {
+        /* Nothing to do */
+        return;
+    }
+    /* Revert to our local time zone */
+    (void) tz_set_tz(LocalTimeZone);
+
+    DSEToday = LocalDSEToday;
+    SysTime = LocalSysTime;
+    FromDSE(DSEToday, &CurYear, &CurMon, &CurDay);
+
+    if (DebugFlag & DB_SWITCH_ZONE) {
+        fprintf(stderr, "TZ exit %s: %04d-%02d-%02d %02d:%02d\n", tz,
+                CurYear, CurMon+1, CurDay, SysTime / 3600, (SysTime/60) % 60);
+    }
+    return;
+}
+
+
+void EnterTimezone(char const *tz)
+{
+    struct tm tm;
+    int y, m, d;
+    time_t t;
+
+    if (CalledEnterTimezone) {
+        fprintf(stderr, "EnterTimezone called twice in a row!!!\n");
+        abort();
+    }
+
+    CalledEnterTimezone = 1;
+
+    if (!tz || !*tz) {
+        /* Stay in local timezone */
+        return;
+    }
+
+    FromDSE(LocalDSEToday, &y, &m, &d);
+    tm.tm_sec   = 0;
+    tm.tm_min   = (LocalSysTime/60) % 60;
+    tm.tm_hour  = LocalSysTime / 3600;
+    tm.tm_mday  = d;
+    tm.tm_mon   = m;
+    tm.tm_year  = y - 1900;
+    tm.tm_wday  = 0;  /* Ignored by mktime */
+    tm.tm_yday  = 0;  /* Ignored by mktime */
+    tm.tm_isdst = -1; /* Information not available */
+
+    t = mktime(&tm);    /* Convert local time to seconds */
+
+    /* Set target timezone */
+    (void) tz_set_tz(tz);
+
+    /* Update our variables */
+    (void) localtime_r(&t, &tm);
+
+    SysTime  = tm.tm_min * 60 + (tm.tm_hour * 3600);
+    CurDay   = tm.tm_mday;
+    CurMon   = tm.tm_mon;
+    CurYear  = tm.tm_year + 1900;
+    /* Adjust DSEToday back by a day if midnight in our time zone requires it */
+    if (SysTime < LocalSysTime) {
+        DSEToday--;
+        FromDSE(DSEToday, &CurYear, &CurMon, &CurDay);
+    }
+
+    if (DebugFlag & DB_SWITCH_ZONE) {
+        fprintf(stderr, "TZ enter %s: %04d-%02d-%02d %02d:%02d\n", tz,
+                CurYear, CurMon+1, CurDay, SysTime / 3600, (SysTime/60) % 60);
+    }
+}
+
 void remove_trailing_newlines(DynamicBuffer *buf)
 {
     char *s = (char *) DBufValue(buf) + DBufLen(buf) - 1;
@@ -224,6 +345,11 @@ static void ensure_satnode_mentions_trigdate(expr_node *node)
 {
     int mentioned = 0;
     char const *str;
+
+    /* Don't bother unless warning_level is 05.00.03 */
+    if (!warning_level("05.00.03")) {
+        return;
+    }
     if (node->type == N_CONSTANT || node->type == N_SHORT_STR) {
         if (node->type == N_CONSTANT) {
             if (node->u.value.type == INT_TYPE) {
@@ -287,6 +413,18 @@ int DoRem(ParsePtr p)
         return r;
     }
 
+    if (trig.tz != NULL && tim.ttime == NO_TIME) {
+        PurgeEchoLine("%s\n", CurLine);
+        FreeTrig(&trig);
+        return E_TZ_NO_AT;
+    }
+
+    /* An empty string for time zone is just a missing time zone */
+    if (trig.tz != NULL && !*trig.tz) {
+        free( (void *) trig.tz);
+        trig.tz = NULL;
+    }
+
     if (trig.complete_through != NO_DATE && !trig.is_todo) {
         PurgeEchoLine("%s\n", CurLine);
         FreeTrig(&trig);
@@ -312,7 +450,9 @@ int DoRem(ParsePtr p)
             PurgeEchoLine("%s\n", "#!P: Cannot purge SATISFY-type reminders");
         }
         PurgeEchoLine("%s\n", CurLine);
+        EnterTimezone(trig.tz);
         r=DoSatRemind(&trig, &tim, p);
+        ExitTimezone(trig.tz);
         if (r) {
             if (r == E_CANT_TRIG && trig.maybe_uncomputable) {
                 r = OK;
@@ -372,7 +512,9 @@ int DoRem(ParsePtr p)
         }
     } else {
         /* Calculate the trigger date */
+        EnterTimezone(trig.tz);
         dse = ComputeTrigger(get_scanfrom(&trig), &trig, &tim, &r, 1);
+        ExitTimezone(trig.tz);
         if (r) {
             if (PurgeMode) {
                 if (!Hush) {
@@ -388,6 +530,9 @@ int DoRem(ParsePtr p)
         }
     }
 
+    /* Adjust trigger date/time to time zone */
+    dse = AdjustTriggerForTimeZone(&trig, dse, &tim);
+
     /* Add to global OMITs if so indicated */
     if (trig.addomit) {
         r = AddGlobalOmit(dse);
@@ -437,6 +582,7 @@ int DoRem(ParsePtr p)
     }
 
     r = OK;
+
     if (ShouldTriggerReminder(&trig, &tim, dse, &err)) {
         /* Filter unwanted events/todos */
         if (todo_filtered(&trig)) {
@@ -479,6 +625,9 @@ int DoRem(ParsePtr p)
             }
             if (tim.ttime != NO_TIME) {
                 PrintJSONKeyPairInt("time", tim.ttime);
+                if (tim.ttime_orig != tim.ttime) {
+                    PrintJSONKeyPairInt("time_in_tz", tim.ttime_orig);
+                }
             }
             if (p->nonconst_expr) {
                 PrintJSONKeyPairInt("nonconst_expr", 1);
@@ -664,11 +813,13 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     trig->omitfunc[0] = 0;
     trig->duration_days = 0;
     trig->eventstart = NO_TIME;
+    trig->eventstart_orig = NO_TIME;
     trig->eventduration = NO_TIME;
     trig->maybe_uncomputable = 0;
     DBufInit(&(trig->tags));
     trig->passthru[0] = 0;
     tim->ttime = NO_TIME;
+    tim->ttime_orig = NO_TIME;
     tim->delta = DefaultTDelta;
     tim->rep   = NO_REP;
     tim->duration = NO_TIME;
@@ -678,6 +829,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     trig->complete_through = NO_DATE;
     trig->adj_for_last = 0;
     trig->infos = NULL;
+    trig->tz = NULL;
 
     int parsing = 1;
     while(parsing) {
@@ -740,6 +892,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             trig->m = m;
             trig->d = d;
             tim->ttime = (tok.val % MINUTES_PER_DAY);
+            tim->ttime_orig = tim->ttime;
             break;
 
         case T_WkDay:
@@ -795,6 +948,7 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             DBufFree(&buf);
             if (tim->ttime != NO_TIME) return E_TIME_TWICE;
             tim->ttime = tok.val;
+            tim->ttime_orig = tok.val;
             r = ParseTimeTrig(s, tim);
             if (r) return r;
             trig->duration_days = ComputeTrigDuration(tim);
@@ -951,6 +1105,22 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             DBufFree(&buf);
             break;
 
+        case T_Tz:
+            if (trig->tz) {
+                return E_TZ_SPECIFIED_TWICE;
+            }
+
+            r = ParseTokenOrQuotedString(s, &buf);
+            if (r != OK) {
+                return r;
+            }
+            trig->tz = StrDup(DBufValue(&buf));
+            if (!trig->tz) {
+                return E_NO_MEM;
+            }
+            DBufFree(&buf);
+            break;
+
         case T_Info:
             r = ParseQuotedString(s, &buf);
             if (r != OK) {
@@ -1018,7 +1188,9 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
             trig->typ = MSG_TYPE;
             if (s->isnested) return E_CANT_NEST_RTYPE;
             if (!WarnedAboutImplicit && !SuppressImplicitRemWarnings) {
-                Wprint(tr("Missing REM type; assuming MSG"));
+                if (warning_level("05.00.03")) {
+                    Wprint(tr("Missing REM type; assuming MSG"));
+                }
                 WarnedAboutImplicit = 1;
             }
             parsing = 0;
@@ -1046,6 +1218,22 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
     }
 
     /* Check for some warning conditions */
+    if (warning_level("06.01.03")) {
+        if (trig->addomit) {
+            if (trig->y == NO_YR || trig->m == NO_MON || trig->d == NO_DAY || trig->rep != NO_REP) {
+                if (trig->scanfrom == NO_SCANFROM) {
+                    if (trig->y == NO_YR && trig->m != NO_MON && trig->d != NO_DAY) {
+                        Wprint(tr("Warning: Consider using SCANFROM -28 with recurring ADDOMIT"));
+                    } else if (trig->m == NO_MON && trig->d != NO_DAY) {
+                        Wprint(tr("Warning: Consider using SCANFROM -7 with recurring ADDOMIT"));
+                    } else {
+                        Wprint(tr("Warning: Consider using SCANFROM with recurring ADDOMIT"));
+                    }
+                }
+            }
+        }
+    }
+
     if (!s->nonconst_expr) {
         if (trig->y != NO_YR && trig->m != NO_MON && trig->d != NO_DAY && trig->until != NO_UNTIL) {
             if (DSE(trig->y, trig->m, trig->d) > trig->until) {
@@ -1069,9 +1257,11 @@ int ParseRem(ParsePtr s, Trigger *trig, TimeTrig *tim)
 
     /* Check that any SCHED / WARN / OMITFUNC functions refer to
        their arguments */
-    check_trigger_function(trig->sched, "SCHED");
-    check_trigger_function(trig->warn, "WARN");
-    check_trigger_function(trig->omitfunc, "OMITFUNC");
+    if (warning_level("05.00.03")) {
+        check_trigger_function(trig->sched, "SCHED");
+        check_trigger_function(trig->warn, "WARN");
+        check_trigger_function(trig->omitfunc, "OMITFUNC");
+    }
     return OK;
 }
 
@@ -1097,6 +1287,7 @@ static int ParseTimeTrig(ParsePtr s, TimeTrig *tim)
             DBufFree(&buf);
             if (tim->ttime != NO_TIME) return E_TIME_TWICE;
             tim->ttime = tok.val;
+            tim->ttime_orig = tok.val;
             break;
 
         case T_Delta:
@@ -1817,6 +2008,7 @@ int DoSatRemind(Trigger *trig, TimeTrig *tt, ParsePtr p)
         } else if (dse == start) {
             if (tt->ttime != NO_TIME) {
                 trig->eventstart = MINUTES_PER_DAY * r + tt->ttime;
+                trig->eventstart_orig = trig->eventstart;
                 if (tt->duration != NO_TIME) {
                     trig->eventduration = tt->duration;
                 }
