@@ -84,8 +84,8 @@ exitfunc(void)
 
         UnsetAllUserFuncs();
         print_expr_nodes_stats();
-        fprintf(ErrFp, "Max expr node evaluations per line: %d\n", MaxExprNodesPerLine);
-        fprintf(ErrFp, "Total expression node evaluations:  %d\n", ExpressionNodesEvaluated);
+        fprintf(ErrFp, "Max expr node evaluations per line: %lu\n", MaxExprNodesPerLine);
+        fprintf(ErrFp, "Total expression node evaluations:  %lu\n", ExpressionNodesEvaluated);
     }
 }
 
@@ -449,7 +449,9 @@ static void DoReminders(void)
 
             default:
                 if (!SuppressImplicitRemWarnings) {
-                    Wprint(tr("Unrecognized command; interpreting as REM"));
+                    if (warning_level("05.00.03")) {
+                        Wprint(tr("Unrecognized command; interpreting as REM"));
+                    }
                     WarnedAboutImplicit = 1;
                 }
                 DestroyParser(&p);
@@ -1453,7 +1455,9 @@ static int DoDebug(ParsePtr p)
             else     DebugFlag &= ~DB_SWITCH_ZONE;
             break;
         default:
-            Wprint(GetErr(M_BAD_DB_FLAG), ch);
+            if (warning_level("05.02.03")) {
+                Wprint(GetErr(M_BAD_DB_FLAG), ch);
+            }
             break;
         }
     }
