@@ -31,6 +31,13 @@ static char const DontEscapeMe[] =
 #include "globals.h"
 #include "protos.h"
 
+/* Call this instead of system() so if called ignores return code,
+   we don't get a compiler warning */
+int system1(char const *cmd)
+{
+    return system(cmd);
+}
+
 /***************************************************************/
 /*                                                             */
 /*  system_to_stderr                                           */
@@ -341,3 +348,10 @@ line_range(int lineno_start, int lineno)
     }
     return buf;
 }
+
+int
+warning_level(char const *which)
+{
+    if (!WarningLevel) return 1;
+    return strcmp(WarningLevel, which) >= 0;
+}
